unit msgbox;

interface

uses
		{$IFDEF WIN32}
			Windows,
		{$ELSE}
     WinTypes, WinProcs,
		{$ENDIF}
	 SysUtils,Classes,Graphics, Controls,Forms, Dialogs, DsgnIntf;

const {These are the button width and height and spacing distances}
	 {$IFDEF WIN32}
		 mcButtonWidth = 46;
		 mcButtonHeight = 17;
		 mcButtonSpacing = 5;
		 mcMessageLimit = 1024;
		 gcFileName = 255;
	 {$ELSE}
		 mcButtonWidth = 40;
		 mcButtonHeight = 14;
		 mcButtonSpacing = 4;
		 mcMessageLimit = 255;
    gcFileName = 80;
	 {$ENDIF}

type

TMessageForm = class(TForm)
	 private
		 procedure HelpButtonClick(Sender: TObject);
	 end;
	 TMessageWave = (mbWave, mbNone, mbDefault, mbConfirm, mbCriticalStop,
									 mbWarning, mbInformation);
	 TIconSource = (ioDefault, ioResource, ioExternal);
	 TTitleOptions = (toDefault, toCustom, toNone);
	 TSymbol = (dgWarning,dgError,dgInformation,dgConfimation,dgNone);
	 TDialogStyle = (dsStayOnTop, dsNormal);
	 {**************************************************************************}
	 { Icon names that appear in the drop down list}
	 TTheIcons =(zzIcon_1,zzIcon_2,zzIcon_3,zzIcon_4,zzIcon_5,zzIcon_6,zzIcon_7,
							 zzIcon_8,zzIcon_9);
  {**************************************************************************}

	 TMsgBox = class(TComponent)
	 private
		 FText,FCustomTitleText : TCaption;
		 FWavFileName : TFileName;
		 FButtons: TMsgDlgButtons;
		 FSymbol: TSymbol;
		 FHelpIndex,FXposition,FYposition : Integer;
		 FUseGlyth : Boolean;
		 FPlayWav : TMessageWave;
		 FAlignment : TAlignment;
		 FFont,FButtonFont : TFont;
		 FCustomIcon : TTheIcons;
		 FIconExternal : TIcon;
		 FTheBtnFocus : TMsgDlgBtn;
		 FTitleOptions : TTitleOptions;
		 FIconSource : TIconSource;
		 FDialogStyle : TDialogStyle;

		 procedure setDialogStyle(value : TDialogStyle);
		 procedure setButtonFont(value : TFont);
		 procedure setIconSource(value : TIconSource);
		 procedure setXposition(value : Integer);
		 procedure setYposition(value : Integer);
		 procedure setTitleOptions(value : TTitleOptions);
		 procedure setTheBtnFocus(value : TMsgDlgBtn);
		 procedure setIconExternal(value : TIcon);
		 procedure setCustomTitleText(value : TCaption);
		 procedure setCustomIcon(value : TTheIcons);
		 procedure setFont(value : TFont);
		 procedure setAlignment(value: TAlignment);
		 procedure setWavFileName(value : TFileName);
		 procedure setPlayWav(value : TMessageWave);
		 procedure setUseGlyth(value : Boolean);
		 procedure SetHelpIndex(Value : Integer);
		 procedure SetButtons(Value: TMsgDlgButtons);
		 procedure SetSymbol(Value: TSymbol);
		 procedure SetText(Value: TCaption);
		 function Max(I, J: Integer): Integer;
		 function GetAveCharSize(Canvas: TCanvas): TPoint;
		 function CreateMessageDialog(const Msg: TCaption; DlgType: TMsgDlgType;
							Buttons: TMsgDlgButtons): TForm;
		 function MyMessageDlg(const Msg: TCaption; DlgType: TMsgDlgType;
							Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;
	 protected

	 public
		 function Execute: Word;
		 constructor Create(AOwner: TComponent); override;
		 destructor Destroy; override;
	 published
		 property DialogStyle : TDialogStyle read FDialogStyle write setDialogStyle default dsStayOnTop;
		 property ButtonFont : TFont read FButtonFont write setButtonFont;
		 property IconSource : TIconSource read FIconSource write setIconSource default ioDefault;
		 property Xposition : Integer read FXposition write SetXPosition default -1;
		 property Yposition : Integer read FYposition write SetYPosition default -1;
		 property TitleOptions : TTitleOptions read FTitleOptions write setTitleOptions default toDefault;
		 property ButtonFocus : TMsgDlgBtn read FTheBtnFocus write setTheBtnFocus default mbOK;
		 property TitleCustomText : TCaption read FCustomTitleText write setCustomTitleText;
		 property IconCustom : TTheIcons read FCustomIcon write setCustomIcon default zzICON_1;
		 property IconExternal : Ticon read FIconExternal write setIconExternal;
		 property TextFont : TFont read FFont write setFont;
		 property TextAlignment : TAlignment read FAlignment write setAlignment default taCenter;
		 property TextMsg: TCaption read FText write SetText;
		 property WavFileName: TFileName read FWavFileName write setWavFileName;
		 property ButtonUseGlyth: Boolean read FUseGlyth write setUseGlyth default True;
		 property PlayWav: TMessageWave read FPlayWav write setPlayWav;
		 property Buttons: TMsgDlgButtons read FButtons write SetButtons;
		 property Symbol: TSymbol read FSymbol write SetSymbol;
		 property HelpIndex: Integer read FHelpIndex write SetHelpIndex default 0;
	end;

{TMsgBoxEditor = class(TDefaultEditor)
	 public
		 procedure ExecuteVerb(Index: Integer); override;
		 function GetVerb(Index: Integer): string; override;
		 function GetVerbCount: Integer; override;
	 end;}

procedure Register;

implementation

uses StdCtrls,ExtCtrls,Consts,MMSystem,Buttons;

constructor TMsgBox.Create(AOwner: TComponent);
begin
	 inherited Create(AOwner);
	 FUseGlyth := True;
	 FAlignment := taCenter;
	 FFont := TFont.Create;
	 FButtonFont := TFont.Create;
	 FIconExternal := TIcon.Create;
  FButtonFont := TFont.Create;
	 FCustomIcon := zzICON_1;
	 FButtons := [mbOK];
	 FTheBtnFocus := mbOK;
	 FTitleOptions:= toDefault;
	 FIconSource := ioDefault;
	 {if these two values equal -1 then the dialog is centered on the screen}
	 FYposition := -1;
	 FXposition := -1;
	 FHelpIndex := 0;
end;
{______________--------------______________________}
{MessageDllg stuff}
procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
	 Application.HelpContext(HelpContext);
end;

function TMsgBox.Max(I, J: Integer): Integer;
begin
  if I > J then Result := I else Result := J;
end;

function TMsgBox.GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
	 for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
	 GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
	 Result.X := Result.X div 52;
end;

function TMsgBox.CreateMessageDialog(const Msg: TCaption; DlgType: TMsgDlgType;
	 Buttons: TMsgDlgButtons): TForm;
const
	 mcHorzMargin = 8;
	 mcVertMargin = 8;
	 mcHorzSpacing = 10;
	 mcVertSpacing = 10;
	 Captions: array[TMsgDlgType] of Word = (SMsgDlgWarning, SMsgDlgError,
		 SMsgDlgInformation, SMsgDlgConfirm, 0);
	 IconIDs: array[TMsgDlgType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
		 IDI_ASTERISK, IDI_QUESTION, nil);
		 {**************************************************}
		 {These Icon Names must match the Icon names in the Resource file}
	 CustomIconIDs : array[TTheIcons] of Pchar = ('ZZICON_1','ZZICON_2','ZZICON_3',
									'ZZICON_4','ZZICON_5','ZZICON_6','ZZICON_7','ZZICON_8',
									'ZZICON_9');
		 {**************************************************}
	 ButtonNames: array[TMsgDlgBtn] of string = (
		 'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'Help');
	 ButtonCaptions: array[TMsgDlgBtn] of Word = (
		 SMsgDlgYes, SMsgDlgNo, SMsgDlgOK, SMsgDlgCancel, SMsgDlgAbort,
		 SMsgDlgRetry, SMsgDlgIgnore, SMsgDlgAll, SMsgDlgHelp);
	 ModalResults: array[TMsgDlgBtn] of Integer = (
		 mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore, mrAll, 0);
	 ButtonTypes: array[TMsgDlgBtn] of TBitBtnKind = (
		bkYes, bkNo, bkOk, bkCancel, bkAbort, bkRetry, bkIgnore, bkAll, bkHelp);
	 TheFormStyle: array[TDialogStyle] of TFormStyle = (fsStayOnTop, fsNormal);

var
	 DialogUnits: TPoint;
	 HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
	 ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
	 IconTextWidth, IconTextHeight, X,I: Integer;
	 B, CancelButton: TMsgDlgBtn;
	 IconID: PChar;
	 TextRect: TRect;
	 temp : array[0..mcMessageLimit] of char;
begin
	 Result := TMessageForm.CreateNew(Application);
	 with Result do
	 begin
		 FormStyle := TheFormStyle[FDialogStyle];
		 BorderStyle := bsDialog;
		 { Get average size for buttons}
		 Canvas.Font := FButtonFont;
		 DialogUnits := GetAveCharSize(Canvas);
		 ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
		 ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
		 ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
		 { Get average size Label}
		 Canvas.Font := FFont;
		 DialogUnits := GetAveCharSize(Canvas);
		 HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
		 VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
		 HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
		 VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
		 SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
		 StrPCopy(temp,Msg);
		 DrawText(Canvas.Handle, temp , -1, TextRect,
			 DT_CALCRECT or DT_WORDBREAK);
		 IconTextWidth := TextRect.Right;
		 IconTextHeight := TextRect.Bottom;
		 case FIconSource of
			 ioDefault : IconID := IconIDs[DlgType];
			 ioResource : IconID := CustomIconIDs[FCustomIcon]
		 end;{case}
		 if IconID <> nil then
		 begin
			 Inc(IconTextWidth, 32 + HorzSpacing);
			 if IconTextHeight < 32 then IconTextHeight := 32;
		 end;
		 ButtonCount := 0;
		 for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
			 if B in Buttons then Inc(ButtonCount);
		 ButtonGroupWidth := 0;
		 if ButtonCount <> 0 then
			 ButtonGroupWidth := ButtonWidth * ButtonCount +
				 ButtonSpacing * (ButtonCount - 1);
		 ClientWidth := Max(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;
		 ClientHeight := IconTextHeight + ButtonHeight + VertSpacing +
			 VertMargin * 2;
		 Left := (Screen.Width div 2) - (Width div 2);
		 Top := (Screen.Height div 2) - (Height div 2);
		 case FTitleOptions Of
			 toDefault : if DlgType <> mtCustom then
											Caption := LoadStr(Captions[DlgType])
									 else
											 Caption := Application.Title;
			 toCustom  : Caption := FCustomTitleText;
			 toNone    : Caption := ''
		 end;{case}
		 if IconID <> nil then
			 with TImage.Create(Result) do
			 begin
				 Name := 'Image';
				 Parent := Result;
				 case FIconSource of
						 ioDefault : Picture.Icon.Handle := LoadIcon(0, IconID);
						 ioResource : Picture.icon.Handle := LoadIcon(hInstance, IconID);
						 ioExternal : Picture.Icon.Handle := FIconExternal.Handle
				 end;{case}
				 SetBounds(HorzMargin, VertMargin, 32, 32);
			 end; {Timage with}
		 with  TLabel.Create(Result) do
		 begin
			 Name := 'Message';
			 Parent := Result;
			 WordWrap := True;
			 AutoSize := False;
			 Alignment := FAlignment;
			 Font := FFont;
			 Caption := Msg;
			 BoundsRect := TextRect;
			 SetBounds(IconTextWidth - TextRect.Right + HorzMargin, VertMargin,
						TextRect.Right, TextRect.Bottom);
			end;
		 if mbCancel in Buttons then CancelButton := mbCancel else
			 if mbNo in Buttons then CancelButton := mbNo else
				 CancelButton := mbOk;
		 X := (ClientWidth - ButtonGroupWidth) div 2;
		 for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
			 if B in Buttons then
				 with TBitBtn.Create(Result) do
				 begin {Button create}
					 Name := ButtonNames[B];
					 Parent := Result;
					 Font := FButtonFont;
					 Caption := LoadStr(ButtonCaptions[B]);
					 ModalResult := ModalResults[B];
					 if FUseGlyth then Kind := ButtonTypes[B];
					 if B = CancelButton then Cancel := True;
					 SetBounds(X, IconTextHeight + VertMargin + VertSpacing,
						 ButtonWidth, ButtonHeight);
					 Inc(X, ButtonWidth + ButtonSpacing);
					 if B = mbHelp then
						 OnClick := TMessageForm(Result).HelpButtonClick;
				 end;{button create with}
			 {Set Button Focus }
			 for I := 0 to ComponentCount -1 do
				if Components[I] is TBitBtn then
					if TBitBtn(Components[I]).Name = ButtonNames[FTheBtnFocus] then
						TBitBtn(Components[I]).TabOrder := 0;
		end;
end;

function TMsgBox.MyMessageDlg(const Msg: TCaption; DlgType: TMsgDlgType;
	 Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;
const
	 Sounds: array[TMessageWave] of Integer = (0,0,MB_OK,
		 MB_ICONQUESTION,mb_IconHand,MB_ICONEXCLAMATION,MB_ICONASTERISK);

var
		s: Array[0..gcFileName] of char;

	begin
	 with CreateMessageDialog(Msg, DlgType, Buttons) do
		 try
			 HelpContext := HelpCtx;
			 if X >= 0 then Left := X;
			 if Y >= 0 then Top := Y;
			 if not (FPlayWav in [mbNone, mbWave]) then
				 MessageBeep(Sounds[FPlayWav])
			 else if FplayWav = mbWave then
        begin
					 StrPCopy(s, FWavFileName);
					 sndPlaySound(s, SND_ASYNC);
				end;
				Result := ShowModal
		 finally
			 Free;
		 end;
end;

destructor TMsgBox.Destroy;
{------------------------------------------------------------------------------}
begin
	 inherited Destroy;
	 FFont.Free;
	 FButtonFont.Free;
	 FIconExternal.Free;
	end;

procedure TMsgBox.setDialogStyle(value : TDialogStyle);
begin
	 FDialogStyle := value;
end;

procedure TMsgBox.setButtonFont(value : TFont);
begin
	FButtonFont.Assign(Value);;
end;

procedure TMsgBox.setIconSource(value : TIconSource);
begin
	FIconSource:= Value;
end;


procedure TMsgBox.setIconExternal(value : TIcon);
begin
	 FIconExternal.Assign(Value);
end;

procedure TMsgBox.setXposition(value : Integer);
begin
		FXposition:= Value
end;

procedure TMsgBox.setYposition(value : Integer);
begin
	 FYposition:= Value
end;

procedure TMsgBox.setTitleOptions(value : TTitleOptions);
begin
	 FTitleOptions:= Value;
end;

procedure TMsgBox.setTheBtnFocus(value : TMsgDlgBtn);
begin
	 FTheBtnFocus:= Value;
end;

procedure TMsgBox.SetButtons(Value: TMsgDlgButtons);
begin
	 FButtons:= Value;
end;

procedure TMsgBox.SetSymbol(Value: TSymbol);
begin
	 FSymbol:= Value;
end;

procedure TMsgBox.SetText(Value: TCaption);
begin
	 FText:= Value;
end;

procedure TMsgBox.SetHelpIndex(Value : Integer);
begin
	 FHelpIndex := Value;
end;

procedure TMsgBox.setPlayWav(value : TMessageWave);
begin
	 FPlayWav := Value;
end;

procedure TMsgBox.setWavFileName(value : TFileName);
begin
	 FWavFileName := Value;
end;

procedure TMsgBox.setUseGlyth(value : Boolean);
begin
	FUseGlyth := Value;
end;

procedure TMsgBox.setAlignment(value: TAlignment);
begin
	 FAlignment := Value;
end;

procedure TMsgBox.setCustomIcon(value : TTheIcons);
begin
	 FCustomIcon := Value;
end;

procedure TMsgBox.setFont(value : TFont);
begin
	 FFont.Assign(Value);
end;

procedure TMsgBox.setCustomTitleText(value : TCaption);
begin
	FCustomTitleText := Value;
end;

function TMsgBox.Execute: Word;
const
	 TheSymbols: array[TSymbol] of TMsgDlgType = (
						mtWarning, mtError,mtInformation,mtConfirmation, mtCustom);

begin
	 Result:= MyMessageDlg(FText, TheSymbols[FSymbol], FButtons, FHelpIndex,
												 FXposition,FYposition);
end;

procedure Register;
begin
	 RegisterComponents('Dialogs', [TMsgBox]);
	 {RegisterComponentEditor(TMsgBox, TMsgBoxEditor); }
end;

{const
	 Verb : string = 'Wav FileName...';

procedure TMsgBoxEditor.ExecuteVerb(Index: Integer);
var
	TheOpenDialog: TOpenDialog;

	begin
	 try
		 TheOpenDialog := TOpenDialog.Create(Application);
		 with TheOpenDialog do
		 begin
			 InitialDir := 'c:\';
			 DefaultExt := 'WAV';
			 Filter := 'Wav Files|*.WAV';
			 if Execute = True then
				begin
					TMsgBox( Component ).WavFileName := FileName;
					Designer.Modified;
				end;
			end;{with}
	 {		finally
			 TheOpenDialog.Free;
	{		end;{try finaly}
{end; {proc}

{function TMsgBoxEditor.GetVerb(Index: Integer): string;
begin
	 Result := Verb
end;

function TMsgBoxEditor.GetVerbCount: Integer;
begin
	 Result := 1
end;   }

end.

